﻿/*
  Autor: Wojciech Domski
  Email: y0yster@o2.pl
  lub
  Email: Wojciech.Domski@gmail.com

  Strona
             Domski.pl
             Wojciech.Domski.pl

  copyright 2014 Wojciech Domski

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Niniejszy program jest wolnym oprogramowaniem; możesz go
    rozprowadzać dalej i/lub modyfikować na warunkach Powszechnej
    Licencji Publicznej GNU, wydanej przez Fundację Wolnego
    Oprogramowania - według wersji 2 tej Licencji lub (według twojego
    wyboru) którejś z późniejszych wersji.

    Niniejszy program rozpowszechniany jest z nadzieją, iż będzie on
    użyteczny - jednak BEZ JAKIEJKOLWIEK GWARANCJI, nawet domyślnej
    gwarancji PRZYDATNOŚCI HANDLOWEJ albo PRZYDATNOŚCI DO OKREŚLONYCH
    ZASTOSOWAŃ. W celu uzyskania bliższych informacji sięgnij do
    Powszechnej Licencji Publicznej GNU.

    Z pewnością wraz z niniejszym programem otrzymałeś też egzemplarz
    Powszechnej Licencji Publicznej GNU (GNU General Public License);
    jeśli nie - napisz do Free Software Foundation, Inc., 59 Temple
    Place, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "OneWire.h"

#include "temerature.h"
#include "MAX7221.h"
#include "filter.h"

OneWire therm1(2);
OneWire therm2(3);

OneWire * thermometer[2] = {&therm1, &therm2};

unsigned char addr[16];
unsigned char * addresses[2] = {&addr[0], &addr[8]};
	
byte data[12];

float temp[2];

int present;
float celsius;

tSinglePoleCoeff filtersTemerature[2];

void initTemp( void)
{
	//initialize DS18B20
	for( int i = 0; i < 2; ++i)
	{
		thermometer[i]->search(addresses[i]);
		thermometer[i]->reset_search();
		delay(250);
	}
	
	//external DS18B20 run first idle measurement
	for( int j = 0; j < 2; ++j)
	{
		getTemp(j);
	}
	
	delay(750);
	
	SinglePoleLowPassInitAll( 0.920, getTemp(0), &filtersTemerature[0]); //temp1
	SinglePoleLowPassInitAll( 0.920, getTemp(1), &filtersTemerature[1]); //temp2
}

float getTemp( int id)
{
	present = thermometer[id]->reset();

	if( present == 0)
	{
		temp[id] = 0;
		return 0.0;
	}
	thermometer[id]->select(addresses[id]);
	thermometer[id]->write(0xBE);
	
	for ( int j = 0; j < 9; j++) {
		data[j] = thermometer[id]->read();
	}

	int16_t raw = (data[1] << 8) | data[0];

	celsius = (float)raw / 16.0;
	temp[id] = celsius;
	
	thermometer[id]->reset();
	thermometer[id]->select(addresses[id]);
	thermometer[id]->write(0x44);        // start conversion, with parasite power off at the end
	
	return celsius;
}



float readTemp( void)
{
	static String str;
	
	for( int i = 0; i < 2; ++i)
	{
		temp[i] = getTemp(i);
		temp[i] = SinglePoleLowPass( temp[i], &filtersTemerature[i]);

		Serial.print("Temperature[");
		str = String(i);
		Serial.print(str.c_str());
		Serial.print( "] = ");
		Serial.println(temp[i]);
	}
}

void showTemp( void)
{
	int tmp;
	int negative = 0;
	for( int i = 0; i < 2; ++i)
	{
		tmp = (int)(temp[i] * 10.0);
		if(tmp < 0)
		{
			tmp *= -1;
			negative = 1;
		}
		
		for( int j = 0; j < 4; ++j)
		{
			//add digit sign
			if( j == 1)
			sendCommand(4-j+i*4, (tmp % 10) | 0x80);
			else
			sendCommand(4-j+i*4, tmp % 10);
			
			tmp /= 10;
			
			if(tmp == 0)
			{
				++j;
				
				//special cases
				//ex. 0.2
				if( j == 1)
				{
					sendCommand(4-j+i*4, (tmp % 10) | 0x80);
					++j;
					
				}

				if( negative == 1)
				{
					sendCommand(4-j+i*4, 0b00001010);
					++j;
				}
				
				clearRest(j, i);
				
				break;
			}
		}
	}
}